package org.classpath.icedtea.pulseaudio;

import java.io.IOException;

import org.classpath.icedtea.pulseaudio.EventLoop;
import org.classpath.icedtea.pulseaudio.PulseAudioVolumeControl;

public class PulseAudioStreamVolumeControl extends PulseAudioVolumeControl {

	@SuppressWarnings("unused")
	private long streamPointer;

	private EventLoop eventLoop;
	private PulseAudioSourceDataLine line;

	static {
		try {
			String library = new java.io.File(".").getCanonicalPath()

			+ java.io.File.separatorChar + System.mapLibraryName("pulse-java");

			System.out.println(library);
			System.load(library);
		} catch (IOException e) {
			assert ("Loading failed".endsWith("library"));
		}
	}

	protected PulseAudioStreamVolumeControl(PulseAudioSourceDataLine line) {
		super(line);
		this.streamPointer = line.getStream().getStreamPointer();
		this.eventLoop = line.getEventLoop();
		this.line = line;
	}

	public synchronized void setValue(float newValue) {
		if (newValue > PulseAudioVolumeControl.MAX_VOLUME
				|| newValue < PulseAudioStreamVolumeControl.MIN_VOLUME) {
			throw new IllegalArgumentException("invalid value");
		}

		if (!line.isMuted()) {
			setStreamVolume(newValue);
		}

		line.setVolume(newValue);
	}

	protected synchronized void setStreamVolume(float newValue) {
		int operationPointer;
		int operationState;
		synchronized (eventLoop.threadLock) {
			operationPointer = native_setValue(newValue);
			operationState = native_getOperationState(operationPointer);
		}
		while (operationState != 1) {
			synchronized (eventLoop.threadLock) {
				operationState = native_getOperationState(operationPointer);
			}
		}

	}

	public synchronized float getValue() {
		return line.getVolume();
	}

	private native int native_getOperationState(int operationState);

	public native int native_setValue(float newValue);
}
