/* org_classpath_icedtea_pulseaudio_EventLoop.c
 Copyright (C) 2008 Red Hat, Inc.

 This file is part of IcedTea.

 IcedTea is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as published by
 the Free Software Foundation, version 2.

 IcedTea is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with IcedTea; see the file COPYING.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301 USA.

 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
 conditions of the GNU General Public License cover the whole
 combination.

 As a special exception, the copyright holders of this library give you
 permission to link this library with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on this library.  If you modify this library, you may extend
 this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version.
 */

#include <pulse/pulseaudio.h>

#include "org_classpath_icedtea_pulseaudio_EventLoop.h"
#include "jni-common.h"

const int PA_ITERATE_BLOCK = 1;
const int PA_ITERATE_NOBLOCK = 0;

static java_context_t* java_context = NULL;

JNIEnv* pulse_thread_env = NULL;

static void context_change_callback(pa_context* context, void* userdata) {
	assert(context);
	assert(userdata == NULL);

	//java_context_t* java_context = (java_context_t*)userdata;
	JNIEnv* env = java_context->env;
	jobject obj = java_context->obj;

	//	printf("context state changed to %d\n", pa_context_get_state(context));

	/* Call the 'update' method in java
	 * to handle all java-side events
	 */
	jclass cls = (*env)->GetObjectClass(env, obj);
	if (cls == NULL) {
		printf("unable to get class of object");
		return;
	}
	jmethodID mid = (*env)->GetMethodID(env, cls, "update", "(I)V");
	if (mid == NULL) {
		printf("unable to get callback method\n");
		return;

	}
	(*env)->CallVoidMethod(env, obj, mid, pa_context_get_state(context));
	return;

}

/*
 * Class:     org_classpath_icedtea_pulseaudio_EventLoop
 * Method:    native_setup
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_classpath_icedtea_pulseaudio_EventLoop_native_1setup
(JNIEnv* env, jobject obj, jstring appName, jstring server) {

	assert(appName != NULL);

	//	printf("native_setup() called\n");
	pa_mainloop *mainloop = pa_mainloop_new();
	assert(mainloop != NULL);
	pa_mainloop_api *mainloop_api = pa_mainloop_get_api(mainloop);
	assert(mainloop != NULL);

	pa_context *context = NULL;

	const char* string_appName;
	string_appName = (*env)->GetStringUTFChars(env, appName, NULL);
	if (string_appName == NULL) {
		return; /* a OutOfMemoryError thrown by vm */
	}
	//	printf("using appName : %s\n", string_appName);
	context = pa_context_new(mainloop_api, string_appName);
	assert(mainloop != NULL);
	(*env)->ReleaseStringUTFChars(env, appName, string_appName);

	obj = (*env)->NewGlobalRef(env, obj);

	java_context = malloc(sizeof(java_context_t));
	java_context->env = env;
	pulse_thread_env = env;
	java_context->obj = obj;

	pa_context_set_state_callback(context, context_change_callback, NULL);

	if (server != NULL) {
		/* obtain the server from the caller */
		const char* string_server = NULL;
		string_server = (*env)->GetStringUTFChars(env, server, NULL);
		if (string_server == NULL) {
			/* error, so clean up */
			(*env)->DeleteGlobalRef(env, java_context->obj);
			pa_context_disconnect(context);
			pa_mainloop_free(mainloop);
			free(java_context);
			return; /* OutOfMemoryError */
		}
		//		printf("About to connect to server: %s\n", string_server);
		pa_context_connect(context, string_server, 0, NULL);
		(*env)->ReleaseStringUTFChars(env, appName, string_server);
	} else {
		//		printf("using default server\n");
		pa_context_connect(context, NULL, 0, NULL);
	}

	setJavaPointer(env, obj, "mainloopPointer", mainloop);
	setJavaPointer(env, obj, "contextPointer", context);
	//	printf("native_setup() returning\n");
	return;

}
/*
 * Class:     org_classpath_icedtea_pulseaudio_EventLoop
 * Method:    native_iterate
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_org_classpath_icedtea_pulseaudio_EventLoop_native_1iterate
(JNIEnv* env, jobject obj, jint timeout) {

	pa_mainloop* mainloop = (pa_mainloop*) getJavaPointer(env, obj, "mainloopPointer");
	assert(mainloop);

	int returnval;

	returnval = pa_mainloop_prepare(mainloop, timeout);
	if ( returnval < 0) {
		return -1;
	}
	returnval = pa_mainloop_poll(mainloop);
	if ( returnval < 0) {
		return -1;
	}
	returnval = pa_mainloop_dispatch(mainloop);
	if ( returnval < 0) {
		return -1;
	}
	return returnval;

}

static void context_drain_complete_callback(pa_context* context, void* userdata) {
	pa_context_disconnect(context);

}

/*
 * Class:     org_classpath_icedtea_pulseaudio_EventLoop
 * Method:    native_shutdown
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_classpath_icedtea_pulseaudio_EventLoop_native_1shutdown
(JNIEnv *env, jobject obj) {

//	printf("native_shutdown() starting\n");

	pa_mainloop* mainloop = (pa_mainloop*) getJavaPointer(env, obj, "mainloopPointer");
	assert(mainloop != NULL);

	pa_context* context = (pa_context*) getJavaPointer(env, obj, "contextPointer");
	assert(context != NULL);

	pa_operation* o = pa_context_drain(context, context_drain_complete_callback, NULL);
	if ( o == NULL) {
		pa_context_disconnect(context);
		pa_mainloop_free(mainloop);
	} else {
		pa_operation_unref(o);
	}

	(*env)->DeleteGlobalRef(env, java_context->obj);

	free(java_context);

	setJavaPointer(env, obj, "mainloopPointer", NULL);
	setJavaPointer(env, obj, "contextPointer", NULL);

	//	printf("native_shutdown() returning\n");

}

static void sink_input_volume_change_complete(pa_context* contest, int success,
		void* userdata) {
	// userdata is the pointer to the int containing the new volume 

	assert(userdata);
	free(userdata);
	assert(success);
	printf("volume change complete\n");

}

static void sink_input_change_volume(pa_context* c,
		const pa_sink_input_info* i, int eol, void* userdata) {
	assert(c);

	// end of list ?
	if (eol) {
		return;
	}

	assert(i);
	assert(userdata);
	int volume = *((int*)userdata);

	int j = 0;
	//	printf("changing sink input volume\n");
	pa_cvolume* new_volume = malloc(sizeof(pa_cvolume));

	//	printf("allocated memory\n");
	new_volume->channels = i->volume.channels;
	//	printf("set the number of channels\n");
	for (j = 0; j < new_volume->channels; j++) {
		new_volume->values[j] = volume;
	}

	//	printf("calling set_sick_input_volume\n");
	pa_operation* o = pa_context_set_sink_input_volume(c, i->index, new_volume,
			sink_input_volume_change_complete, new_volume);
	if (o != NULL) {
		pa_operation_unref(o);
	}

	//	printf("done setup for changing volume\n");
}

/*
 * Class:     org_classpath_icedtea_pulseaudio_EventLoop
 * Method:    native_set_sink_volume
 * Signature: (JI)V
 */
JNIEXPORT void JNICALL Java_org_classpath_icedtea_pulseaudio_EventLoop_native_1set_1sink_1volume
(JNIEnv* env, jobject obj, jlong streamPointer, jint volume) {
	int* new_volume = malloc(sizeof(int));
	*new_volume = volume;
	int stream_id = pa_stream_get_index((pa_stream*) convertJavaLongToPointer(streamPointer));
	pa_context_get_sink_input_info((pa_context*) getJavaPointer(env, obj,"contextPointer") ,stream_id,sink_input_change_volume, new_volume);
	return;
}

