package org.classpath.icedtea.pulseaudio;

import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.TargetDataLine;

import junit.framework.JUnit4TestAdapter;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class PulseAudioTargetDataLineTest {

	public static junit.framework.Test suite() {
		return new JUnit4TestAdapter(PulseAudioTargetDataLineTest.class);
	}

	private Mixer mixer;
	private TargetDataLine targetDataLine;

	@Before
	public void setUp() throws LineUnavailableException {
		Mixer.Info[] mixerInfos = AudioSystem.getMixerInfo();
		Mixer.Info wantedMixerInfo = null;
		for (Mixer.Info mixerInfo : mixerInfos) {
			if (mixerInfo.getName().contains("PulseAudio")) {
				wantedMixerInfo = mixerInfo;
			}
		}
		Assert.assertNotNull(wantedMixerInfo);
		mixer = AudioSystem.getMixer(wantedMixerInfo);
		Assert.assertNotNull(mixer);
		mixer.open();

	}

	@Test
	public void testOpenClose() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));
		Assert.assertNotNull(targetDataLine);
		targetDataLine.open();
		targetDataLine.close();
	}

	@Test
	public void testOpenEvents() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));
		Assert.assertNotNull(targetDataLine);

		LineListener openListener = new LineListener() {
			private int calledCount = 0;

			@Override
			public void update(LineEvent event) {
				Assert.assertEquals(LineEvent.Type.OPEN, event.getType());
				calledCount++;
				Assert.assertEquals(1, calledCount);
			}

		};

		targetDataLine.addLineListener(openListener);
		targetDataLine.open();
		targetDataLine.removeLineListener(openListener);
		targetDataLine.close();
		
	}

	@Test
	public void testCloseEvents() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));
		Assert.assertNotNull(targetDataLine);

		LineListener closeListener = new LineListener() {
			private int calledCount = 0;

			@Override
			public void update(LineEvent event) {
				Assert.assertEquals(LineEvent.Type.CLOSE, event.getType());
				calledCount++;
				Assert.assertEquals(1, calledCount);
			}

		};


		targetDataLine.open();
		targetDataLine.addLineListener(closeListener);
		targetDataLine.close();
		targetDataLine.removeLineListener(closeListener);
		
	}
	
	
	@After
	public void tearDown() {
		mixer.close();
	}

}
